package main

import (
	"bufio"
	"flag"
	"fmt"
	"io"
	"log"
	"os"
	"strings"
	"time"

	"github.com/influxdata/cron"
)

func rel_from_now(s *bufio.Scanner) int {
	now_t := time.Now()

	// seed with a large number that rtcwake can still handle
	cur_rel_wake_est_s := 268435455
	//we need to only get the first 5 fields otherwise
	//the cron library will try to parse the rest too
	READ_FIELDS := 5

	for s.Scan() {
		fields := strings.Split(strings.TrimSpace(s.Text()), " ")
		l := strings.Join(fields[:READ_FIELDS], " ")
		p, e := cron.ParseUTC(l)
		if e == nil {
			entry_est_t, e := p.Next(now_t)
			if e != nil {
				log.Fatal(e)
			}
			entry_estimate_s := int(entry_est_t.Sub(now_t).Seconds())
			if entry_estimate_s < cur_rel_wake_est_s {
				cur_rel_wake_est_s = entry_estimate_s
			}
			log.Print(entry_estimate_s)
		} else {
			log.Print(e)
		}
	}
	return cur_rel_wake_est_s;
}

func main() {
	verbose  := flag.Bool("v", false, "display log output")
	absolute := flag.Bool("a", false, "display unix absolute time for next cron job")
	human    := flag.Bool("h", false, "display time in human readable format")
	flag.Parse()

	if !*verbose {
		log.SetOutput(io.Discard)
		log.SetFlags(0)
	}

	s := bufio.NewScanner(os.Stdin)
	rel_wake_est_s := rel_from_now(s)

	if *absolute {
		now_t := time.Now()
		abstime := now_t.Add(time.Duration(rel_wake_est_s) * time.Second)
		if *human {
			fmt.Println(abstime.Format(time.RFC3339))
		} else {
			fmt.Println(abstime.Unix())
		}
		os.Exit(0);
	}

	if *human {
		fmt.Println((time.Duration(rel_wake_est_s) * time.Second).String())
	} else {
		fmt.Println(rel_wake_est_s)
	}

}
